package com.xydl.cac.service.impl;

import com.xydl.cac.entity.NiecPoint;
import com.xydl.cac.entity.NiecSensor;
import com.xydl.cac.model.ConditionModel;
import com.xydl.cac.model.SensorDetail;
import com.xydl.cac.repository.NiecSensorRepository;
import com.xydl.cac.service.NiecPointService;
import com.xydl.cac.service.NiecSensorService;
import com.xydl.cac.util.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;

@Service
@Slf4j
@Transactional(rollbackFor = Exception.class)
public class NiecSensorServiceImpl implements NiecSensorService {

    @Resource
    NiecSensorRepository repository;
    @Resource
    NiecPointService pointService;
    @Resource
    private JdbcTemplate jdbcTemplate;

    @Override
    public List<NiecSensor> listAll() {
        List<NiecSensor> list = repository.findAll();
        return list;
    }

    @Override
    public Page<NiecSensor> list(int pageNum, int pageSize) {
        PageRequest request = PageRequest.of(pageNum, pageSize);
        Page<NiecSensor> result = repository.findAll(request);
        return result;
    }

    @Override
    public List<NiecSensor> getTree() {
        List<NiecSensor> list = this.listAll();
        List<NiecSensor> result = new ArrayList<>();
        for (NiecSensor item : list) {
            boolean done = false;
            if (item.getParentId() != null) {
                for (NiecSensor parent : list) {
                    if (parent.getId().intValue() == item.getParentId().intValue()) {
                        if (StringUtils.isNotBlank(item.getTableName())) {
                            parent.addChild(item);
                        }
                        done = true;
                        break;
                    }
                }
            }
            if (!done) {
                result.add(item);
            }
        }
        return result;
    }

    @Override
    public SensorDetail getDetail(ConditionModel model) throws Exception {
        Optional<NiecSensor> optional = repository.findById(model.getId());
        if (!optional.isPresent()) {
            throw new Exception("未找到该装置");
        }
        NiecSensor sensor = optional.get();
        if (StringUtils.isBlank(sensor.getDtimeFieldName())) {
            throw new Exception("该装置缺少采样时间字段名信息");
        }
        if (StringUtils.isBlank(sensor.getTableName())) {
            throw new Exception("该装置缺少table_name信息");
        }
        if (StringUtils.isBlank(sensor.getDevidFieldName())) {
            throw new Exception("该装置缺少sensor_id的字段名信息");
        }
        if (sensor.getDevId() == null) {
            throw new Exception("该装置缺少dev_id信息");
        }
        List<NiecPoint> points = pointService.getList(sensor.getId(), 2);
        if (CollectionUtils.isEmpty(points)) {
            throw new Exception("该装置缺少业务点表配置信息");
        }

        SensorDetail result = this.getData(sensor, points, model);
        return result;
    }

    public SensorDetail getData(NiecSensor sensor, List<NiecPoint> points, ConditionModel model) {
        SensorDetail result = new SensorDetail();
        result.setSensor(sensor);
        List<NiecPoint> pointList = new ArrayList<>();

        String sqlFrom = " FROM " + sensor.getTableName();
        String sqlWhere = " WHERE " + sensor.getDevidFieldName() + "=" + sensor.getDevId();
        if (model.getStartTime() != null) {
            sqlWhere = sqlWhere + " AND " + sensor.getDtimeFieldName() + " >= '" + DateUtil.format(model.getStartTime()) + "'";
        }
        if (model.getEndTime() != null) {
            sqlWhere = sqlWhere + " AND " + sensor.getDtimeFieldName() + " < '" + DateUtil.format(model.getEndTime()) + "'";
        }
        String sqlOrder = " ORDER BY " + sensor.getDtimeFieldName() + " DESC";
        String sqlLimit = "";
        if (model.getPageNum() != null && model.getPageSize() != null) {
            result.setPageNum(model.getPageNum());
            result.setPageSize(model.getPageSize());
            sqlLimit = String.format(" LIMIT %d,%d", (model.getPageNum() - 1) * model.getPageSize(), model.getPageSize());
        }

        String sqlSelect = "SELECT DATE_FORMAT(" + sensor.getDtimeFieldName() + ", '%Y-%m-%d %H:%i:%s') as acquisitionTime";
        for (NiecPoint point : points) {
            if (StringUtils.isNotBlank(point.getField())) {
                sqlSelect = sqlSelect + ", " + point.getField();
                pointList.add(point);
            }
        }

        String sqlCount = "SELECT count(*)" + sqlFrom + sqlWhere;
        String sql = sqlSelect + sqlFrom + sqlWhere + sqlOrder + sqlLimit;
        log.info(sql);

        Long count = jdbcTemplate.queryForObject(sqlCount, Long.class);
        List<Map<String, Object>> content = jdbcTemplate.queryForList(sql);

        for (NiecPoint point : pointList) {
            List<Object> dataList = new ArrayList<>();
            for (Map<String, Object> item : content) {
                Object value = item.get(point.getField());
                dataList.add(value);
            }
            point.setData(dataList);
        }

        result.setTotal(count);
        result.setPoints(pointList);
        result.setContent(content);
        return result;
    }
}
